Type: Package
Package: rjd3bench
Title: Temporal Disaggregation and Benchmarking in 'JDemetra+' 3.x
Version: 3.1.2
Authors@R: c(
      person("Jean", "Palate", email = "palatejean@gmail.com", role = "aut"),
      person("Corentin", "Lemasson", email ="corentin.lemasson@nbb.be", role = c("aut", "cre")),
      person("Tanguy", "Barthelemy", email ="tanguy.barthelemy@insee.fr", role = c("ctb", "art")),
      person("Nikolina", "Rizanovska", email ="nikolina.rizanovska@gov.si", role = "ctb")
  )
Description: Interface to 'JDemetra+' 3.x (<https://github.com/jdemetra>) 
    time series analysis software. It provides a variety of methods for 
    temporal disaggregation & interpolation, benchmarking, reconciliation 
    and calendarization. It incorporates statistical methods described in 
    the latest European Statistical System (ESS) guidelines on temporal 
    disaggregation, benchmarking, and reconciliation (2018 edition). The package 
    implements highly efficient algorithms for fast and reliable computation.
License: EUPL
URL: https://github.com/rjdverse/rjd3bench,
        https://rjdverse.github.io/rjd3bench/
BugReports: https://github.com/rjdverse/rjd3bench/issues
Depends: R (>= 4.1.0)
Imports: rJava (>= 1.0-6), rjd3toolkit (>= 3.7.1), RProtoBuf (>=
        0.4.20), stats, graphics
Encoding: UTF-8
RoxygenNote: 7.3.3
SystemRequirements: Java (>= 21)
Collate: 'utils.R' 'adl.R' 'benchmark.R' 'calendarization.R'
        'deprecated.R' 'mbdenton.R' 'print.R' 'tempdisagg.R' 'zzz.R'
LazyData: TRUE
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-04 09:12:03 UTC; lemasso
Author: Jean Palate [aut],
  Corentin Lemasson [aut, cre],
  Tanguy Barthelemy [ctb, art],
  Nikolina Rizanovska [ctb]
Maintainer: Corentin Lemasson <corentin.lemasson@nbb.be>
Repository: CRAN
Date/Publication: 2026-05-04 09:50:02 UTC
