# Generated by roxygen2: do not edit by hand

S3method(plot,JD3_ADLDISAGG_RSLTS)
S3method(plot,JD3_INTERP_RSLTS)
S3method(plot,JD3_MBDENTON_RSLTS)
S3method(plot,JD3_TEMPDISAGGI_RSLTS)
S3method(plot,JD3_TEMPDISAGG_RSLTS)
S3method(print,JD3_ADLDISAGG_RSLTS)
S3method(print,JD3_INTERPRAW_RSLTS)
S3method(print,JD3_INTERP_RSLTS)
S3method(print,JD3_MBDENTON_RSLTS)
S3method(print,JD3_TEMPDISAGGI_RSLTS)
S3method(print,JD3_TEMPDISAGGRAW_RSLTS)
S3method(print,JD3_TEMPDISAGG_RSLTS)
S3method(summary,JD3_ADLDISAGG_RSLTS)
S3method(summary,JD3_INTERPRAW_RSLTS)
S3method(summary,JD3_INTERP_RSLTS)
S3method(summary,JD3_MBDENTON_RSLTS)
S3method(summary,JD3_TEMPDISAGGI_RSLTS)
S3method(summary,JD3_TEMPDISAGGRAW_RSLTS)
S3method(summary,JD3_TEMPDISAGG_RSLTS)
export(adl_disaggregation)
export(calendarization)
export(cholette)
export(cubicspline)
export(denton)
export(denton_modelbased)
export(denton_raw)
export(grp)
export(multivariatecholette)
export(temporal_disaggregation)
export(temporal_disaggregation_raw)
export(temporal_interpolation)
export(temporal_interpolation_raw)
export(temporaldisaggregation)
export(temporaldisaggregationI)
import(RProtoBuf)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(rJava,.jarray)
importFrom(rJava,.jcall)
importFrom(rJava,.jcast)
importFrom(rJava,.jcastToArray)
importFrom(rJava,.jclass)
importFrom(rJava,.jevalArray)
importFrom(rJava,.jinstanceof)
importFrom(rJava,.jnew)
importFrom(rJava,.jnull)
importFrom(rJava,.jpackage)
importFrom(rJava,is.jnull)
importFrom(stats,end)
importFrom(stats,frequency)
importFrom(stats,is.ts)
importFrom(stats,start)
importFrom(stats,ts)
importFrom(stats,ts.plot)
