% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmark.R
\name{cholette}
\alias{cholette}
\title{Benchmarking by means of the Cholette Method}
\usage{
cholette(
  s,
  t,
  rho = 1,
  lambda = 1,
  bias = c("None", "Additive", "Multiplicative"),
  conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
  obsposition = 1L
)
}
\arguments{
\item{s}{A preliminary series. It must be of the same class as \code{t}.}

\item{t}{The low-frequency aggregation constraint. It must be either a \code{"ts"} object or a numeric vector.}

\item{rho}{Numeric. A smoothing parameter whose value must lie between 0 and 1.
See the package vignette for more information on the choice of the \code{rho} parameter.}

\item{lambda}{Numeric. The adjustment model parameter. Typical choices include \code{lambda = 1} for proportional benchmarking, \code{lambda = 0} for additive benchmarking, and \code{lambda = 0.5} with \code{rho = 0} for the naive pro-rating method.
See the package vignette for more information on the choice of the \code{lambda} parameter.}

\item{bias}{Character. Specifies the bias-correction factor. By default, no systematic bias is considered. Other options are: "Additive" and "Multiplicative". See vignette for more details.
See the package vignette for more information on the other alternatives.}

\item{conversion}{A character string specifying the conversion mode, typically \code{"Sum"} (the default) or \code{"Average"}. Other options are: \code{"Last"}, \code{"First"} and \code{"UserDefined"}.}

\item{obsposition}{An integer specifying the position of the observations of the low-frequency constraint within the benchmarked series (e.g. the 7th month of the year).
This argument is used only when \code{conversion = "UserDefined"}.}
}
\value{
A \code{"ts"} object with the benchmarked series is returned.
}
\description{
Cholette method is based on a benchmarking methodology developed at
Statistics Canada. It is a generalized method relying on the principle of
movement preservation that encompasses other benchmarking methods.
The Denton method (both the AFD and PFD variants), as well as the naive
pro-rating method, emerge as particular cases of the Cholette method.
This method has been widely used for the purpose of benchmarking seasonally
adjusted series among others.
}
\examples{
\dontshow{if (rjd3toolkit::get_java_version() >= rjd3toolkit::minimal_java_version) withAutoprint(\{ # examplesIf}
ym_true <- rjd3toolkit::Retail$RetailSalesTotal
yq_true <- rjd3toolkit::aggregate(ym_true, 4)
Y_full <- rjd3toolkit::aggregate(ym_true, 1)

Y <- window(Y_full, end = c(2009,1)) # say no benchmark yet for the year 2010
xm <- ym_true + rnorm(n = length(ym_true), mean = -5000, sd = 10000)
xq <- rjd3toolkit::aggregate(xm, 4)

# Proportional benchmarking with a bias and some recommended value of rho for
# monthly and quarterly series respectively
cholette(s = xm, t = Y, rho = 0.9, lambda = 1, bias = "Multiplicative")
cholette(s = xq, t = Y, rho = 0.729, lambda = 1, bias = "Multiplicative")

# Proportional benchmarking with no bias
xm_no_bias <- ym_true + rnorm(n = length(ym_true), mean = 0, sd = 10000)
cholette(s = xm_no_bias, t = Y, rho = 0.9, lambda = 1)

# Additive benchmarking
cholette(s = xm, t = Y, rho = 0.9, lambda = 0, bias = "Additive")

# Denton PFD
cholette(s = xm, t = Y, rho = 1, lambda = 1)

# Pro-rating
cholette(s = xm, t = Y, rho = 0, lambda = 0.5)
\dontshow{\}) # examplesIf}
}
\references{
Quenneville, B., Fortier S., Chen Z.-G., Latendresse E. (2006).
Recent Developments in Benchmarking to Annual Totals in X12-ARIMA and at
Statistics Canada. Statistics Canada, Working paper of the Time Series
Research and Analysis Centre.
}
\seealso{
For more information, see the vignette:

\code{utils::browseVignettes()}, e.g. \code{browseVignettes(package = "rjd3bench")}
}
