% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{deprecated-rjd3bench}
\alias{deprecated-rjd3bench}
\alias{temporaldisaggregation}
\title{Deprecated Functions}
\usage{
temporaldisaggregation(
  series,
  constant = TRUE,
  trend = FALSE,
  indicators = NULL,
  model = c("Ar1", "Rw", "RwAr1"),
  freq = 4L,
  conversion = c("Sum", "Average", "Last", "First", "UserDefined"),
  conversion.obsposition = 1L,
  rho = 0,
  rho.fixed = FALSE,
  rho.truncated = 0,
  zeroinitialization = FALSE,
  diffuse.algorithm = c("SqrtDiffuse", "Diffuse", "Augmented"),
  diffuse.regressors = FALSE
)
}
\arguments{
\item{series, constant, trend, indicators, model, freq, conversion, conversion.obsposition, rho, rho.fixed, rho.truncated, zeroinitialization, diffuse.algorithm, diffuse.regressors}{Parameters.}
}
\value{
Return the same value as either function that replaces it.
}
\description{
This function is deprecated. Use the function \code{temporal_disaggregation()} or \code{temporal_interpolation()} instead.
}
